@@ -170,8 +170,8 @@ static int dump_sched_info(int pid, ThreadCoreEntry *tc)
 
 	errno = 0;
 	ret = getpriority(PRIO_PROCESS, pid);
-	if (errno) {
-		pr_perror("Can't get nice for %d", pid);
+	if (ret == -1 && errno) {
+		pr_perror("Can't get nice for %d ret %d", pid, ret);
 		return -1;
 	}
 
@@ -184,10 +184,11 @@ static int dump_sched_info(int pid, ThreadCoreEntry *tc)
 
 struct cr_imgset *glob_imgset;
 
-static int collect_fds(pid_t pid, struct parasite_drain_fd *dfds)
+static int collect_fds(pid_t pid, struct parasite_drain_fd **dfds)
 {
 	struct dirent *de;
 	DIR *fd_dir;
+	int size = 0;
 	int n;
 
 	pr_info("\n");
@@ -203,13 +204,20 @@ static int collect_fds(pid_t pid, struct parasite_drain_fd *dfds)
 		if (dir_dots(de))
 			continue;
 
-		if (n > PARASITE_MAX_FDS - 1)
-			return -ENOMEM;
+		if (sizeof(struct parasite_drain_fd) + sizeof(int) * (n + 1) > size) {
+			struct parasite_drain_fd *t;
 
-		dfds->fds[n++] = atoi(de->d_name);
+			size += PAGE_SIZE;
+			t = xrealloc(*dfds, size);
+			if (!t)
+				return -1;
+			*dfds = t;
+		}
+
+		(*dfds)->fds[n++] = atoi(de->d_name);
 	}
 
-	dfds->nr_fds = n;
+	(*dfds)->nr_fds = n;
 	pr_info("Found %d file descriptors\n", n);
 	pr_info("----------------------------------------\n");
 
@@ -307,9 +315,9 @@ static int dump_task_rlimits(int pid, TaskRlimitsEntry *rls)
 	int res;
 
 	for (res = 0; res <rls->n_rlimits ; res++) {
-		struct rlimit lim;
+		struct rlimit64 lim;
 
-		if (prlimit(pid, res, NULL, &lim)) {
+		if (syscall(__NR_prlimit64, pid, res, NULL, &lim)) {
 			pr_perror("Can't get rlimit %d", res);
 			return -1;
 		}
@@ -522,7 +530,7 @@ static int get_task_futex_robust_list(pid_t pid, ThreadCoreEntry *info)
 	int ret;
 
 	ret = syscall(SYS_get_robust_list, pid, &head, &len);
-	if (ret == -ENOSYS) {
+	if (ret < 0 && errno == ENOSYS) {
 		/*
 		 * If the kernel says get_robust_list is not implemented, then
 		 * check whether set_robust_list is also not implemented, in
@@ -534,7 +542,8 @@ static int get_task_futex_robust_list(pid_t pid, ThreadCoreEntry *info)
 		 * implemented, in which case it will return -EINVAL because
 		 * len should be greater than zero.
 		 */
-		if (syscall(SYS_set_robust_list, NULL, 0) != -ENOSYS)
+		ret = syscall(SYS_set_robust_list, NULL, 0);
+		if (ret == 0 || (ret < 0 && errno != ENOSYS))
 			goto err;
 
 		head = NULL;
@@ -1207,7 +1216,7 @@ static int dump_one_task(struct pstree_item *item)
 		if (!dfds)
 			goto err;
 
-		ret = collect_fds(pid, dfds);
+		ret = collect_fds(pid, &dfds);
 		if (ret) {
 			pr_err("Collect fds (pid: %d) failed with %d\n", pid, ret);
 			goto err;
@@ -1262,7 +1271,7 @@ static int dump_one_task(struct pstree_item *item)
 		goto err_cure_imgset;
 	}
 
-	ret = parasite_check_aios(parasite_ctl, &vmas); /* FIXME -- merge with above */
+	ret = parasite_collect_aios(parasite_ctl, &vmas); /* FIXME -- merge with above */
 	if (ret) {
 		pr_err("Failed to check aio rings (pid: %d)\n", pid);
 		goto err_cure_imgset;
@@ -1418,12 +1427,14 @@ static int cr_pre_dump_finish(struct list_head *ctls, int ret)
 {
 	struct parasite_ctl *ctl, *n;
 
-	pstree_switch_state(root_item,
-			ret ? TASK_ALIVE : opts.final_state);
+	pstree_switch_state(root_item, TASK_ALIVE);
 	free_pstree(root_item);
 
 	timing_stop(TIME_FROZEN);
 
+	if (ret < 0)
+		goto err;
+
 	pr_info("Pre-dumping tasks' memory\n");
 	list_for_each_entry_safe(ctl, n, ctls, pre_list) {
 		struct page_xfer xfer;
@@ -1432,14 +1443,14 @@ static int cr_pre_dump_finish(struct list_head *ctls, int ret)
 		timing_start(TIME_MEMWRITE);
 		ret = open_page_xfer(&xfer, CR_FD_PAGEMAP, ctl->pid.virt);
 		if (ret < 0)
-			break;
+			goto err;
 
 		ret = page_xfer_dump_pages(&xfer, ctl->mem_pp, 0);
 
 		xfer.close(&xfer);
 
 		if (ret)
-			break;
+			goto err;
 
 		timing_stop(TIME_MEMWRITE);
 
@@ -1448,9 +1459,12 @@ static int cr_pre_dump_finish(struct list_head *ctls, int ret)
 		parasite_cure_local(ctl);
 	}
 
-	if (irmap_predump_run())
+	if (irmap_predump_run()) {
 		ret = -1;
+		goto err;
+	}
 
+err:
 	if (disconnect_from_page_server())
 		ret = -1;
 
@@ -1689,6 +1703,15 @@ int cr_dump_tasks(pid_t pid)
 			goto err;
 	}
 
+	/*
+	 * It may happen that a process has completed but its files in
+	 * /proc/PID/ are still open by another process. If the PID has been
+	 * given to some newer thread since then, we may be unable to dump
+	 * all this.
+	 */
+	if (dead_pid_conflict())
+		goto err;
+
 	/* MNT namespaces are dumped after files to save remapped links */
 	if (dump_mnt_namespaces() < 0)
 		goto err;
